@@ -504,7 +504,9 @@ static response pv_list(lvmetad_state *s, request r)
 		return res; /* FIXME error reporting */
 
 	/* The response field */
-	res.cft->root = make_text_node(res.cft, "response", "OK", NULL, NULL);
+	if (!(res.cft->root = make_text_node(res.cft, "response", "OK", NULL, NULL)))
+		return res; /* FIXME doomed */
+
 	cn_pvs = make_config_node(res.cft, "physical_volumes", NULL, res.cft->root);
 
 	lock_pvid_to_pvmeta(s);
@@ -718,9 +720,9 @@ static response vg_lookup(lvmetad_state *s, request r)
 	struct dm_config_node *metadata, *n;
 	struct vg_info *info;
 	response res = { 0 };
-
 	const char *uuid = daemon_request_str(r, "uuid", NULL);
 	const char *name = daemon_request_str(r, "name", NULL);
+	int count = 0;
 
 	buffer_init( &res.buffer );
 
@@ -734,17 +736,23 @@ static response vg_lookup(lvmetad_state *s, request r)
 
 		lock_vgid_to_metadata(s);
 		if (name && !uuid)
-			uuid = dm_hash_lookup(s->vgname_to_vgid, name);
+			uuid = dm_hash_lookup_with_count(s->vgname_to_vgid, name, &count);
 		else if (uuid && !name)
 			name = dm_hash_lookup(s->vgid_to_vgname, uuid);
 		unlock_vgid_to_metadata(s);
 
+		if (name && uuid && (count > 1)) {
+			DEBUGLOG(s, "vg_lookup name %s vgid %s found %d vgids",
+				 name, uuid, count);
+			return daemon_reply_simple("multiple", "reason = %s", "Multiple VGs found with same name", NULL);
+		}
+
 		if (!uuid || !name)
 			return reply_unknown("VG not found");
 
 	} else {
 		char *name_lookup = dm_hash_lookup(s->vgid_to_vgname, uuid);
-		char *uuid_lookup = dm_hash_lookup(s->vgname_to_vgid, name);
+		char *uuid_lookup = dm_hash_lookup_with_val(s->vgname_to_vgid, name, uuid, strlen(uuid) + 1);
 
 		/* FIXME: comment out these sanity checks when not testing */
 
@@ -827,8 +835,8 @@ static int vg_remove_if_missing(lvmetad_state *s, const char *vgid, int update_p
 enum update_pvid_mode { UPDATE_ONLY, REMOVE_EMPTY, MARK_OUTDATED };
 
 /* You need to be holding the pvid_to_vgid lock already to call this. */
-static int update_pvid_to_vgid(lvmetad_state *s, struct dm_config_tree *vg,
-			       const char *vgid, int mode)
+static int _update_pvid_to_vgid(lvmetad_state *s, struct dm_config_tree *vg,
+				const char *vgid, int mode)
 {
 	struct dm_config_node *pv;
 	struct dm_hash_table *to_check;
@@ -915,7 +923,7 @@ static int remove_metadata(lvmetad_state *s, const char *vgid, int update_pvids)
 	name_lookup = dm_hash_lookup(s->vgid_to_vgname, vgid);
 	outdated_pvs_lookup = dm_hash_lookup(s->vgid_to_outdated_pvs, vgid);
 	if (name_lookup)
-		vgid_lookup = dm_hash_lookup(s->vgname_to_vgid, name_lookup);
+		vgid_lookup = dm_hash_lookup_with_val(s->vgname_to_vgid, name_lookup, vgid, strlen(vgid) + 1);
 
 	/* remove hash table mappings */
 
@@ -924,13 +932,13 @@ static int remove_metadata(lvmetad_state *s, const char *vgid, int update_pvids)
 	dm_hash_remove(s->vgid_to_vgname, vgid);
 	dm_hash_remove(s->vgid_to_outdated_pvs, vgid);
 	if (name_lookup)
-		dm_hash_remove(s->vgname_to_vgid, name_lookup);
+		dm_hash_remove_with_val(s->vgname_to_vgid, name_lookup, vgid, strlen(vgid) + 1);
 
 	unlock_vgid_to_metadata(s);
 
 	/* update_pvid_to_vgid will clear/free the pvid_to_vgid hash */
-	if (update_pvids)
-		update_pvid_to_vgid(s, meta_lookup, "#orphan", 0);
+	if (update_pvids && meta_lookup)
+		_update_pvid_to_vgid(s, meta_lookup, "#orphan", 0);
 
 	/* free the unmapped data */
 
@@ -1004,8 +1012,8 @@ static void _purge_metadata(lvmetad_state *s, const char *arg_name, const char *
 	lock_pvid_to_vgid(s);
 	remove_metadata(s, arg_vgid, 1);
 
-	if ((rem_vgid = dm_hash_lookup(s->vgname_to_vgid, arg_name))) {
-		dm_hash_remove(s->vgname_to_vgid, arg_name);
+	if ((rem_vgid = dm_hash_lookup_with_val(s->vgname_to_vgid, arg_name, arg_vgid, strlen(arg_vgid) + 1))) {
+		dm_hash_remove_with_val(s->vgname_to_vgid, arg_name, arg_vgid, strlen(arg_vgid) + 1);
 		dm_free(rem_vgid);
 	}
 	unlock_pvid_to_vgid(s);
@@ -1048,7 +1056,7 @@ static int _update_metadata_new_vgid(lvmetad_state *s,
 	/*
 	 * Temporarily orphan the PVs in the old metadata.
 	 */
-	if (!update_pvid_to_vgid(s, old_meta, "#orphan", 0)) {
+	if (!_update_pvid_to_vgid(s, old_meta, "#orphan", 0)) {
 		ERROR(s, "update_metadata_new_vgid failed to move PVs for %s old_vgid %s", arg_name, old_vgid);
 		abort_daemon = 1;
 		goto ret;
@@ -1072,7 +1080,7 @@ static int _update_metadata_new_vgid(lvmetad_state *s,
 	dm_config_destroy(old_meta);
 	old_meta = NULL;
 
-	dm_hash_remove(s->vgname_to_vgid, arg_name);
+	dm_hash_remove_with_val(s->vgname_to_vgid, arg_name, old_vgid, strlen(old_vgid) + 1);
 	dm_hash_remove(s->vgid_to_vgname, old_vgid);
 	dm_free((char *)old_vgid);
 	old_vgid = NULL;
@@ -1093,7 +1101,7 @@ static int _update_metadata_new_vgid(lvmetad_state *s,
 		goto out;
 	}
 
-	if (!dm_hash_insert(s->vgname_to_vgid, arg_name, new_vgid_dup)) {
+	if (!dm_hash_insert_allow_multiple(s->vgname_to_vgid, arg_name, new_vgid_dup, strlen(new_vgid_dup) + 1)) {
 		ERROR(s, "update_metadata_new_vgid out of memory for vgid hash insert for %s %s", arg_name, new_vgid);
 		abort_daemon = 1;
 		goto out;
@@ -1102,7 +1110,7 @@ static int _update_metadata_new_vgid(lvmetad_state *s,
 	/*
 	 * Reassign PVs based on the new metadata.
 	 */
-	if (!update_pvid_to_vgid(s, new_meta, new_vgid, 1)) {
+	if (!_update_pvid_to_vgid(s, new_meta, new_vgid, 1)) {
 		ERROR(s, "update_metadata_new_name failed to update PVs for %s %s", arg_name, new_vgid);
 		abort_daemon = 1;
 		goto out;
@@ -1162,7 +1170,7 @@ static int _update_metadata_new_name(lvmetad_state *s,
 	/*
 	 * Temporarily orphan the PVs in the old metadata.
 	 */
-	if (!update_pvid_to_vgid(s, old_meta, "#orphan", 0)) {
+	if (!_update_pvid_to_vgid(s, old_meta, "#orphan", 0)) {
 		ERROR(s, "update_metadata_new_name failed to move PVs for old_name %s %s", old_name, arg_vgid);
 		abort_daemon = 1;
 		goto ret;
@@ -1187,7 +1195,7 @@ static int _update_metadata_new_name(lvmetad_state *s,
 	old_meta = NULL;
 
 	dm_hash_remove(s->vgid_to_vgname, arg_vgid);
-	dm_hash_remove(s->vgname_to_vgid, old_name);
+	dm_hash_remove_with_val(s->vgname_to_vgid, old_name, arg_vgid, strlen(arg_vgid) + 1);
 	dm_free((char *)old_name);
 	old_name = NULL;
 
@@ -1207,7 +1215,7 @@ static int _update_metadata_new_name(lvmetad_state *s,
 		goto out;
 	}
 
-	if (!dm_hash_insert(s->vgname_to_vgid, new_name, arg_vgid_dup)) {
+	if (!dm_hash_insert_allow_multiple(s->vgname_to_vgid, new_name, arg_vgid_dup, strlen(arg_vgid_dup) + 1)) {
 		ERROR(s, "update_metadata_new_name out of memory for vgid hash insert for %s %s", new_name, arg_vgid);
 		abort_daemon = 1;
 		goto out;
@@ -1216,7 +1224,7 @@ static int _update_metadata_new_name(lvmetad_state *s,
 	/*
 	 * Reassign PVs based on the new metadata.
 	 */
-	if (!update_pvid_to_vgid(s, new_meta, arg_vgid, 1)) {
+	if (!_update_pvid_to_vgid(s, new_meta, arg_vgid, 1)) {
 		ERROR(s, "update_metadata_new_name failed to update PVs for %s %s", new_name, arg_vgid);
 		abort_daemon = 1;
 		goto out;
@@ -1276,13 +1284,13 @@ static int _update_metadata_add_new(lvmetad_state *s, const char *new_name, cons
 		goto out;
 	}
 
-	if (!dm_hash_insert(s->vgname_to_vgid, new_name, new_vgid_dup)) {
+	if (!dm_hash_insert_allow_multiple(s->vgname_to_vgid, new_name, new_vgid_dup, strlen(new_vgid_dup) + 1)) {
 		ERROR(s, "update_metadata_add_new out of memory for vgid hash insert for %s %s", new_name, new_vgid);
 		abort_daemon = 1;
 		goto out;
 	}
 
-	if (!update_pvid_to_vgid(s, new_meta, new_vgid, 1)) {
+	if (!_update_pvid_to_vgid(s, new_meta, new_vgid, 1)) {
 		ERROR(s, "update_metadata_add_new failed to update PVs for %s %s", new_name, new_vgid);
 		abort_daemon = 1;
 		goto out;
@@ -1325,9 +1333,9 @@ out_free:
  * pvid and ret_old_seq are both NULL.
  */
 
-static int update_metadata(lvmetad_state *s, const char *arg_name, const char *arg_vgid,
-			   struct dm_config_node *new_metadata, int64_t *ret_old_seq,
-			   const char *pvid)
+static int _update_metadata(lvmetad_state *s, const char *arg_name, const char *arg_vgid,
+			    struct dm_config_node *new_metadata, int *ret_old_seq,
+			    const char *pvid)
 {
 	struct dm_config_tree *old_meta = NULL;
 	struct dm_config_tree *new_meta = NULL;
@@ -1343,6 +1351,13 @@ static int update_metadata(lvmetad_state *s, const char *arg_name, const char *a
 	int needs_repair = 0;
 	int abort_daemon = 0;
 	int retval = 0;
+	int count = 0;
+
+	if (!arg_vgid || !arg_name) {
+		ERROR(s, "update_metadata missing args arg_vgid %s arg_name %s pvid %s",
+		      arg_vgid ?: "none", arg_name ?: "none", pvid ?: "none");
+		return 0;
+	}
 
 	DEBUGLOG(s, "update_metadata begin arg_vgid %s arg_name %s pvid %s",
 		 arg_vgid, arg_name, pvid ?: "none");
@@ -1357,7 +1372,7 @@ static int update_metadata(lvmetad_state *s, const char *arg_name, const char *a
 
 	lock_vgid_to_metadata(s);
 	arg_name_lookup = dm_hash_lookup(s->vgid_to_vgname, arg_vgid);
-	arg_vgid_lookup = dm_hash_lookup(s->vgname_to_vgid, arg_name);
+	arg_vgid_lookup = dm_hash_lookup_with_val(s->vgname_to_vgid, arg_name, arg_vgid, strlen(arg_vgid) + 1);
 
 	/*
 	 * A new VG when there is no existing record of the name or vgid args.
@@ -1376,7 +1391,7 @@ static int update_metadata(lvmetad_state *s, const char *arg_name, const char *a
 	 * A lookup of the name arg was successful in finding arg_vgid_lookup,
 	 * but that resulting vgid doesn't match the arg_vgid.
 	 */
-	if (arg_vgid_lookup && arg_vgid && strcmp(arg_vgid_lookup, arg_vgid)) {
+	if (arg_vgid_lookup && strcmp(arg_vgid_lookup, arg_vgid)) {
 		if (arg_name_lookup) {
 			/*
 			 * This shouldn't happen.
@@ -1389,7 +1404,22 @@ static int update_metadata(lvmetad_state *s, const char *arg_name, const char *a
 		}
 
 		new_vgid = arg_vgid;
-		old_vgid = dm_hash_lookup(s->vgname_to_vgid, arg_name);
+		old_vgid = dm_hash_lookup_with_count(s->vgname_to_vgid, arg_name, &count);
+
+		/*
+		 * FIXME: this ensures that arg_name maps to only one existing
+		 * VG (old_vgid), because if it maps to multiple vgids, then we
+		 * don't know which one should get the new vgid (arg_vgid).  If
+		 * this function was given both the existing name and existing
+		 * vgid to identify the VG, then this wouldn't be a problem.
+		 * But as it is now, the vgid arg to this function is the new
+		 * vgid and the existing VG is specified only by name.
+		 */
+		if (old_vgid && (count > 1)) {
+			ERROR(s, "update_metadata arg_vgid %s arg_name %s found %d vgids for name",
+			      arg_vgid, arg_name, count);
+			old_vgid = NULL;
+		}
 
 		if (!old_vgid) {
 			/* This shouldn't happen. */
@@ -1417,7 +1447,7 @@ static int update_metadata(lvmetad_state *s, const char *arg_name, const char *a
 	 * A lookup of the vgid arg was successful in finding arg_name_lookup,
 	 * but that resulting name doesn't match the arg_name.
 	 */
-	if (arg_name_lookup && arg_name && strcmp(arg_name_lookup, arg_name)) {
+	if (arg_name_lookup && strcmp(arg_name_lookup, arg_name)) {
 		if (arg_vgid_lookup) {
 			/*
 			 * This shouldn't happen.
@@ -1457,6 +1487,14 @@ static int update_metadata(lvmetad_state *s, const char *arg_name, const char *a
 	 * An existing VG has unchanged name and vgid.
 	 */
 	if (!new_vgid && !new_name) {
+		if (!arg_vgid_lookup || !arg_name_lookup) {
+			/* This shouldn't happen. */
+			ERROR(s, "update_metadata arg_vgid %s arg_name %s missing lookups vgid %s name %s",
+			      arg_vgid ?: "none", arg_name ?: "none", arg_vgid_lookup ?: "none", arg_name_lookup ?: "none");
+			needs_repair = 1;
+			goto update;
+		}
+
 		if (strcmp(arg_name_lookup, arg_name)) {
 			/* This shouldn't happen. */
 			ERROR(s, "update_metadata arg_vgid %s arg_name %s mismatch arg_name_lookup %s",
@@ -1466,23 +1504,11 @@ static int update_metadata(lvmetad_state *s, const char *arg_name, const char *a
 		}
 
 		if (strcmp(arg_vgid_lookup, arg_vgid)) {
-			/*
-			 * This shouldn't usually happen, but could when
-			 * disks are moved (or filters are changed?)
-			 *
-			 * FIXME: two different VGs with different uuids, on
-			 * different PVs could have the same name, e.g. disks
-			 * are moved, filters are changed.  Possible solution:
-			 * purge all info for these VGs and return an error
-			 * that causes the command to go to disk.  Would this
-			 * only happen from pv_found?
-			 * Until there's a proper solution, ignore this VG.
-			 */
-			ERROR(s, "update_metadata ignore VG with duplicate name %s vgid %s pvid %s existing vgid %s",
-			      arg_name, arg_vgid, pvid ?: "none", arg_vgid_lookup);
-			unlock_vgid_to_metadata(s);
-			retval = 0;
-			goto out;
+			/* This shouldn't happen.  Two VGs with the same name is handled above. */
+			ERROR(s, "update_metadata arg_vgid %s arg_name %s mismatch arg_vgid_lookup %s",
+			      arg_vgid, arg_name, arg_vgid_lookup);
+			needs_repair = 1;
+			goto update;
 		}
 
 		/* old_vgid == arg_vgid, and old_name == arg_name */
@@ -1580,7 +1606,8 @@ static int update_metadata(lvmetad_state *s, const char *arg_name, const char *a
 	if (pvid && needs_repair) {
 		ERROR(s, "update_metadata ignore inconsistent metadata on PV %s seqno %d for %s %s seqno %d",
 		      pvid, new_seq, arg_vgid, arg_name, old_seq);
-		DEBUGLOG_cft(s, "OLD: ", old_meta->root);
+		if (old_meta)
+			DEBUGLOG_cft(s, "OLD: ", old_meta->root);
 		DEBUGLOG_cft(s, "NEW: ", new_metadata);
 		retval = 0;
 		goto out;
@@ -1592,7 +1619,8 @@ static int update_metadata(lvmetad_state *s, const char *arg_name, const char *a
 	if (!pvid && needs_repair) {
 		ERROR(s, "update_metadata inconsistent with cache for vgid %s and name %s",
 		      arg_vgid, arg_name);
-		DEBUGLOG_cft(s, "OLD: ", old_meta->root);
+		if (old_meta)
+			DEBUGLOG_cft(s, "OLD: ", old_meta->root);
 		DEBUGLOG_cft(s, "NEW: ", new_metadata);
 		abort_daemon = 1;
 		retval = 0;
@@ -1644,7 +1672,7 @@ static int update_metadata(lvmetad_state *s, const char *arg_name, const char *a
 		DEBUGLOG_cft(s, "OLD: ", old_meta->root);
 		DEBUGLOG_cft(s, "NEW: ", new_metadata);
 		lock_pvid_to_vgid(s);
-		update_pvid_to_vgid(s, old_meta, arg_vgid, MARK_OUTDATED);
+		_update_pvid_to_vgid(s, old_meta, arg_vgid, MARK_OUTDATED);
 		unlock_pvid_to_vgid(s);
 	}
 
@@ -1769,9 +1797,9 @@ static int update_metadata(lvmetad_state *s, const char *arg_name, const char *a
 	 * The PVs in the VG may have changed in the new metadata, so
 	 * temporarily orphan all of the PVs in the existing VG.
 	 * The PVs that are still in the VG will be reassigned to this
-	 * VG below by the next call to update_pvid_to_vgid().
+	 * VG below by the next call to _update_pvid_to_vgid().
 	 */
-	if (!update_pvid_to_vgid(s, old_meta, "#orphan", 0)) {
+	if (!_update_pvid_to_vgid(s, old_meta, "#orphan", 0)) {
 		ERROR(s, "update_metadata failed to move PVs for %s %s", arg_name, arg_vgid);
 		unlock_vgid_to_metadata(s);
 		unlock_pvid_to_vgid(s);
@@ -1809,7 +1837,7 @@ static int update_metadata(lvmetad_state *s, const char *arg_name, const char *a
 	 * be newly mapped to this vgid, and previous PVs that
 	 * remain in the VG will be remapped to the VG again.
 	 */
-	if (!update_pvid_to_vgid(s, new_meta, arg_vgid, 1)) {
+	if (!_update_pvid_to_vgid(s, new_meta, arg_vgid, 1)) {
 		ERROR(s, "update_metadata failed to update PVs for %s %s", arg_name, arg_vgid);
 		abort_daemon = 1;
 		retval = 0;
@@ -2081,10 +2109,10 @@ static response pv_found(lvmetad_state *s, request r)
 	uint64_t arg_device_lookup = 0;
 	uint64_t new_device = 0;
 	uint64_t old_device = 0;
-	int64_t arg_seqno = -1;
-	int64_t old_seqno = -1;
-	int64_t vg_status_seqno = -1;
-	int64_t changed = 0;
+	int arg_seqno = -1;
+	int old_seqno = -1;
+	int vg_status_seqno = -1;
+	int changed = 0;
 
 	/*
 	 * New input values.
@@ -2381,9 +2409,9 @@ static response pv_found(lvmetad_state *s, request r)
 	 * name of the VG so we can check if the VG is complete.
 	 */
 	if (arg_vgmeta) {
-		DEBUGLOG(s, "pv_found pvid %s has VG %s %s seqno %ld", arg_pvid, arg_name, arg_vgid, arg_seqno);
+		DEBUGLOG(s, "pv_found pvid %s has VG %s %s seqno %d", arg_pvid, arg_name, arg_vgid, arg_seqno);
 
-		if (!update_metadata(s, arg_name, arg_vgid, arg_vgmeta, &old_seqno, arg_pvid)) {
+		if (!_update_metadata(s, arg_name, arg_vgid, arg_vgmeta, &old_seqno, arg_pvid)) {
 			ERROR(s, "Cannot use VG metadata for %s %s from PV %s on %" PRIu64,
 			      arg_name, arg_vgid, arg_pvid, arg_device);
 		}
@@ -2465,11 +2493,11 @@ static response pv_found(lvmetad_state *s, request r)
 
 	return daemon_reply_simple("OK",
 				   "status = %s", vg_status,
-				   "changed = %d", changed,
+				   "changed = " FMTd64, (int64_t) changed,
 				   "vgid = %s", arg_vgid ? arg_vgid : "#orphan",
 				   "vgname = %s", arg_name ? arg_name : "#orphan",
-				   "seqno_before = %"PRId64, old_seqno,
-				   "seqno_after = %"PRId64, vg_status_seqno,
+				   "seqno_before = " FMTd64, (int64_t) old_seqno,
+				   "seqno_after = " FMTd64, (int64_t) vg_status_seqno,
 				   NULL);
 
  nomem:
@@ -2542,7 +2570,7 @@ static response vg_update(lvmetad_state *s, request r)
 		/* TODO defer metadata update here; add a separate vg_commit
 		 * call; if client does not commit, die */
 
-		if (!update_metadata(s, vgname, vgid, metadata, NULL, NULL)) {
+		if (!_update_metadata(s, vgname, vgid, metadata, NULL, NULL)) {
 			ERROR(s, "vg_update failed: metadata update failed");
 			reply_fail("vg_update: failed metadata update");
 			goto fail;
@@ -2588,8 +2616,8 @@ static response set_global_info(lvmetad_state *s, request r)
 
 static response get_global_info(lvmetad_state *s, request r)
 {
-	return daemon_reply_simple("OK", "global_invalid = %d",
-					 (s->flags & GLFL_INVALID) ? 1 : 0,
+	return daemon_reply_simple("OK", "global_invalid = " FMTd64,
+					 (int64_t)((s->flags & GLFL_INVALID) ? 1 : 0),
 					 NULL);
 }
 
@@ -2617,6 +2645,11 @@ use_name:
 	if (!(uuid = dm_hash_lookup(s->vgname_to_vgid, name)))
 		goto out;
 
+	/* 
+	 * FIXME: if we only have the name and multiple VGs have that name,
+	 * then invalidate each of them.
+	 */
+
 	if (!(vg = dm_hash_lookup(s->vgid_to_metadata, uuid)))
 		goto out;
 vers:
