@@ -17,14 +17,15 @@
 #include "config.h"
 
 typedef int (*lock_resource_fn) (struct cmd_context * cmd, const char *resource,
-				 uint32_t flags, struct logical_volume *lv);
+				 uint32_t flags, const struct logical_volume *lv);
 typedef int (*query_resource_fn) (const char *resource, int *mode);
 
 typedef void (*fin_lock_fn) (void);
 typedef void (*reset_lock_fn) (void);
 
-#define LCK_PRE_MEMLOCK	0x00000001	/* Is memlock() needed before calls? */
-#define LCK_CLUSTERED	0x00000002
+#define LCK_PRE_MEMLOCK			0x00000001	/* Is memlock() needed before calls? */
+#define LCK_CLUSTERED			0x00000002
+#define LCK_SUPPORTS_REMOTE_QUERIES	0x00000004
 
 struct locking_type {
 	uint32_t flags;
