@@ -20,6 +20,8 @@
 #define SECURITY_LEVEL 0
 #define PV_MIN_SIZE_KB 512
 
+enum dev_ext_e;
+
 void init_verbose(int level);
 void init_silent(int silent);
 void init_test(int level);
@@ -27,7 +29,7 @@ void init_md_filtering(int level);
 void init_fwraid_filtering(int level);
 void init_pvmove(int level);
 void init_full_scan_done(int level);
-void init_external_device_info_source(unsigned src);
+void init_external_device_info_source(enum dev_ext_e src);
 void init_obtain_device_list_from_udev(int device_list_from_udev);
 void init_trust_cache(int trustcache);
 void init_debug(int level);
@@ -38,6 +40,7 @@ void init_lockingfailed(int level);
 void init_security_level(int level);
 void init_mirror_in_sync(int in_sync);
 void init_dmeventd_monitor(int reg);
+void init_disable_dmeventd_monitoring(int disable);
 void init_background_polling(int polling);
 void init_ignore_suspended_devices(int ignore);
 void init_ignore_lvm_mirrors(int scan);
@@ -51,6 +54,7 @@ void init_detect_internal_vg_cache_corruption(int detect);
 void init_retry_deactivation(int retry);
 
 void set_cmd_name(const char *cmd_name);
+const char *get_cmd_name(void);
 void set_sysfs_dir_path(const char *path);
 
 int test_mode(void);
@@ -59,7 +63,7 @@ int fwraid_filtering(void);
 int pvmove_mode(void);
 int full_scan_done(void);
 int obtain_device_list_from_udev(void);
-unsigned external_device_info_source(void);
+enum dev_ext_e external_device_info_source(void);
 int trust_cache(void);
 int verbose_level(void);
 int silent_mode(void);
