@@ -11,8 +11,13 @@
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 
+SKIP_WITH_LVMETAD=1
+SKIP_WITH_CLVMD=1
+
 . lib/inittest
 
+aux prepare_dmeventd
+
 #
 # TODO:
 # lvm2app is not yet capable to respect many lvm.conf options
@@ -25,20 +30,21 @@
 # gdb -ex r --args python FULL_PATH/lvm2/test/api/python_lvm_unit.py -v TestLvm.test_lv_active_inactive
 
 #Locate the python binding library to use.
-python_lib=$(find $abs_top_builddir -name lvm.so)
-# Unable to test python bindings if library not available
-test -n "$python_lib" || skip
+if [[ -n $abs_top_builddir ]]; then
+  python_lib=$(find $abs_top_builddir -name lvm.so)
+  # Unable to test python bindings if library not available
+  test -n "$python_lib" || skip "lvm2-python-libs not built"
 
-test -e LOCAL_CLVMD && skip
-test -e LOCAL_LVMETAD && skip
-
-aux prepare_dmeventd
+  export PYTHONPATH=$(dirname $python_lib):$PYTHONPATH
+elif rpm -q lvm2-python-libs &>/dev/null; then
+  true
+else
+  skip "lvm2-python-libs neither built nor installed"
+fi
 
 #If you change this change the unit test case too.
 aux prepare_pvs 6
 
-export PYTHONPATH=$(dirname $python_lib):$PYTHONPATH
-
 #Setup which devices the unit test can use.
 export PY_UNIT_PVS=$(cat DEVICES)
 
