@@ -14,6 +14,8 @@
 
 # 2 pvmove LVs in 2 VGs (1 per VG)
 
+SKIP_WITH_CLVMD=1
+
 . lib/inittest
 
 aux prepare_pvs 4 30
@@ -31,6 +33,7 @@ test_pvmove_resume() {
 	lvcreate -an -Zn -l30 -n $lv1 $vg1
 
 	aux delay_dev "$dev3" 0 1000 $(get first_extent_sector "$dev3"):
+	test -e HAVE_DM_DELAY || { lvremove -f $vg $vg1; return 0; }
 	aux delay_dev "$dev4" 0 1000 $(get first_extent_sector "$dev4"):
 
 	pvmove -i5 "$dev1" &
@@ -245,8 +248,6 @@ pvmove_bg_single() {
 	LVM_TEST_TAG="kill_me_$PREFIX" pvmove -b "$dev2"
 }
 
-test -e LOCAL_CLVMD && skip
-
 test_pvmove_resume lvchange_single
 test_pvmove_resume lvchange_all
 test_pvmove_resume vgchange_single
